@@ -4,14 +4,14 @@
 # --------------------------------------
 
 # information to display for different objects in first line of output
-print_info <- function(x, ...) UseMethod("print_info")
+print_info <- function(object, ...) UseMethod("print_info")
 
 # information to display for regression fits
-print_info.reg_fit_mediation <- function(x, ...) {
-  prefix <- if (is_robust(x)) "Robust mediation" else "Mediation"
-  if (x$robust == "median") postfix <- "via median regression"
+print_info.reg_fit_mediation <- function(object, ...) {
+  prefix <- if (is_robust(object)) "Robust mediation" else "Mediation"
+  if (object$robust == "median") postfix <- "via median regression"
   else {
-    errors <- switch(x$family, student = " with t errors",
+    errors <- switch(object$family, student = " with t errors",
                      skewnormal = " with skew-normal errors",
                      skewt = " with skew-t errors",
                      select = " with selection of error distribution")
@@ -21,16 +21,16 @@ print_info.reg_fit_mediation <- function(x, ...) {
 }
 
 # information to display for covariance matrix fits
-print_info.cov_fit_mediation <- function(x, ...) {
-  prefix <- if (is_robust(x)) "Robust mediation" else "Mediation"
+print_info.cov_fit_mediation <- function(object, ...) {
+  prefix <- if (is_robust(object)) "Robust mediation" else "Mediation"
   cat(sprintf("%s model fit via covariance matrix\n", prefix))
 }
 
 # information to display for bootstrap tests
-print_info.boot_test_mediation <- function(x, ...) {
+print_info.boot_test_mediation <- function(object, ...) {
   # type of test and model fit
-  fit <- x$fit
-  if (inherits(x$fit, "reg_fit_mediation")) {
+  fit <- object$fit
+  if (inherits(object$fit, "reg_fit_mediation")) {
     prefix <- if (fit$robust == "MM") "Robust bootstrap" else "Bootstrap"
     if (fit$robust == "median") postfix <- " via median regression"
     else {
@@ -48,18 +48,17 @@ print_info.boot_test_mediation <- function(x, ...) {
     postfix <- ""
   }
   # use plural for multiple mediators
-  nr_indirect <- length(x$fit$x) * length(x$fit$m)
-  plural <- if (nr_indirect == 1L) "" else "s"
+  plural <- if (is.null(fit$model) || fit$model == "simple") "" else "s"
   # return message
   cat(sprintf("%s test%s for indirect effect%s%s\n",
               prefix, plural, plural, postfix))
 }
 
 # information to display for sobel tests
-print_info.sobel_test_mediation <- function(x, ...) {
+print_info.sobel_test_mediation <- function(object, ...) {
   # type of test and model fit
-  fit <- x$fit
-  if (inherits(x$fit, "reg_fit_mediation")) {
+  fit <- object$fit
+  if (inherits(object$fit, "reg_fit_mediation")) {
     prefix <- if (fit$robust == "MM") "Robust normal" else "Normal"
     if (fit$robust == "median") postfix <- " via median regression"
     else {
@@ -80,37 +79,42 @@ print_info.sobel_test_mediation <- function(x, ...) {
 }
 
 # information on variables in summary of mediation analysis
-print_info.summary_fit_mediation <- function(x, ...) {
+print_info.summary_fit_mediation <- function(object, ...) {
   # initializations
-  p_x <- length(x$x)
-  p_m <- length(x$m)
-  nr_indirect <- p_x * p_m
-  have_covariates <- length(x$covariates) > 0L
+  p_x <- length(object$x)
+  p_m <- length(object$m)
+  have_simple <- is.null(object$model) || object$model == "simple"
+  have_covariates <- length(object$covariates) > 0L
+  # in case of multiple mediators, print information on type of mediation model
+  if (p_m > 1L) {
+    prefix <- switch(object$model, parallel = "Parallel", serial = "Serial")
+    cat(prefix, "multiple mediator model\n\n")
+  }
   # print information on variables
-  if (nr_indirect == 1L) {
-    cat(sprintf("x = %s\n", x$x))
-    cat(sprintf("y = %s\n", x$y))
-    cat(sprintf("m = %s\n", x$m))
+  if (have_simple) {
+    cat(sprintf("x = %s\n", object$x))
+    cat(sprintf("y = %s\n", object$y))
+    cat(sprintf("m = %s\n", object$m))
   } else {
     width <- max(nchar(p_x), nchar(p_m)) + 1L
-    if (p_x == 1L) cat(sprintf(paste0("%-", width, "s = %s\n"), "x", x$x))
+    if (p_x == 1L) cat(sprintf(paste0("%-", width, "s = %s\n"), "x", object$x))
     else {
       x_labels <- paste0("x", seq_len(p_x))
-      cat(sprintf(paste0("%-", width, "s = %s\n"), x_labels, x$x), sep = "")
+      cat(sprintf(paste0("%-", width, "s = %s\n"), x_labels, object$x), sep = "")
     }
-    cat(sprintf(paste0("%-", width, "s = %s\n"), "y", x$y))
-    if (p_m == 1L) cat(sprintf(paste0("%-", width, "s = %s\n"), "m", x$m))
+    cat(sprintf(paste0("%-", width, "s = %s\n"), "y", object$y))
+    if (p_m == 1L) cat(sprintf(paste0("%-", width, "s = %s\n"), "m", object$m))
     else {
       m_labels <- paste0("m", seq_len(p_m))
-      cat(sprintf(paste0("%-", width, "s = %s\n"), m_labels, x$m), sep = "")
+      cat(sprintf(paste0("%-", width, "s = %s\n"), m_labels, object$m), sep = "")
     }
   }
   if (have_covariates) {
     cat("\nCovariates:\n")
-    print(x$covariates, quote = FALSE)
+    print(object$covariates, quote = FALSE)
   }
   # print sample size
-  cat(sprintf("\nSample size: %d\n", x$n))
+  cat(sprintf("\nSample size: %d\n", object$n))
 }
 
 
@@ -119,11 +123,32 @@ print.fit_mediation <- function(x, info = TRUE, ...) {
   # print information on type of model fit
   if (isTRUE(info)) print_info(x, ...)
   # print estimated effects
-  cat("\nEffects:\n")
-  print(coef(x), ...)
+  if (is.null(x$model) || x$model == "simple") {
+    cat("\nEffects:\n")
+    print(coef(x), ...)
+  } else {
+    # some initializations
+    p_x <- length(x$x)
+    p_m <- length(x$m)
+    # print effects
+    cat("\na paths:\n")
+    print(x[["a"]], ...)
+    cat(sprintf("\nb path%s:\n", if (p_m == 1L) "" else "s"))
+    print(x[["b"]], ...)
+    if (!is.null(x[["d"]])) {
+      cat(sprintf("\nd path%s:\n", if (p_m > 2L) "s" else ""))
+      print(x[["d"]], ...)
+    }
+    cat(sprintf("\nTotal effect%s:\n", if (p_x == 1L) "" else "s"))
+    print(x[["total"]], ...)
+    cat(sprintf("\nDirect effect%s:\n", if (p_x == 1L) "" else "s"))
+    print(x[["direct"]], ...)
+    cat("\nIndirect effects:\n")
+    print(x[["indirect"]], ...)
+  }
   # print information on how contrasts are computed
   # (if there are no contrasts, nothing is printed)
-  print_contrast_info(x, prefix = TRUE)
+  print_contrast_info(x)
   # return object invisibly
   invisible(x)
 }
@@ -157,25 +182,42 @@ print.cov_ML <- function(x, ...) {
 #' @export
 print.boot_test_mediation <- function(x, digits = max(3, getOption("digits")-3),
                                       info = TRUE, ...) {
+  # initializations
+  names_x <- x$fit$x
+  p_x <- length(names_x)
+  names_m <- x$fit$m
+  p_m <- length(names_m)
+  model <- x$fit$model
+  have_simple <- is.null(model) || model == "simple"
   # print information on type of model fit
   if (isTRUE(info)) print_info(x, ...)
-  # initializations
-  m <- x$fit$m
-  nr_indirect <- length(x$fit$x) * length(m)
   # print indirect effects
-  plural <- if (nr_indirect == 1L) "" else "s"
+  plural <- if (have_simple) "" else "s"
   cat(sprintf("\nIndirect effect%s of x on y:\n", plural))
-  # extract indirect effect
-  ab <- cbind(Data = x$fit$ab, Boot = x$ab)
-  if (nr_indirect == 1L) rownames(ab) <- m
-  # extract confidence interval
-  ci <- if (nr_indirect == 1L) t(x$ci) else x$ci
-  colnames(ci) <- c("Lower", "Upper")
+  # extract indirect effect and confidence interval
+  indirect <- cbind(Data = x$fit$indirect, Boot = x$indirect,
+                    if (have_simple) t(x$ci) else x$ci)
+  if (have_simple) rownames(indirect) <- names_m
+  # replace complicated row names with shorter labels
+  if (p_m > 1L && (p_x > 1L || model == "serial")) {
+    labels <- get_indirect_labels(p_m, model = model)
+    if (p_x == 1L) rownames(indirect)[1L + seq_along(labels)] <- labels
+    else {
+      replace <- grep("->", rownames(indirect))
+      replacement_names <- unlist(lapply(names_x, paste, labels, sep = "_"),
+                                  use.names = FALSE)
+      rownames(indirect)[replace] <- replacement_names
+    }
+  } else labels <- NULL
   # combine and print
-  print(cbind(ab, ci), digits = digits, ...)
+  print(indirect, digits = digits, ...)
+  # print information on indirect effect paths for models with multiple
+  # independent variables and multiple mediators, or serial multiple mediators
+  # (if we have a different model, nothing is printed)
+  print_indirect_info(x$fit, labels = labels)
   # print information on how contrasts are computed
   # (if there are no contrasts, nothing is printed)
-  print_contrast_info(x$fit)
+  print_contrast_info(x$fit, labels = labels)
   # print additional information
   cat("---\nLevel of confidence: ", format(100 * x$level), " %\n", sep = "")
   cat(sprintf("\nNumber of bootstrap replicates: %d\n", x$R))
@@ -190,7 +232,7 @@ print.sobel_test_mediation <- function(x, digits = max(3, getOption("digits")-3)
   if (isTRUE(info)) print_info(x, ...)
   # print indirect effect
   cat("\nIndirect effect of x on y:\n")
-  ab <- cbind(x$fit$ab, x$se, x$statistic, x$p_value)
+  ab <- cbind(x$fit$indirect, x$se, x$statistic, x$p_value)
   m <- x$fit$m
   cn <- switch(x$alternative, twosided = "Pr(>|z|)",
                less = "Pr(<z)", greater = "Pr(>z)")
@@ -300,7 +342,7 @@ get_outlier_info <- function(outliers, digits = max(3, getOption("digits")-3)) {
     # information that there is no clear outlier and minimum robustness weight
     msg <- paste("No potential outliers with weight < ",
           formatC(outliers$threshold, digits = max(2, digits-3), width = 1),
-          " detected. The minimum weight is ",
+          " detected.\nThe minimum weight is ",
           formatC(min(outliers$weights), digits = max(2, digits-3), width = 1),
           ".\n", sep = "")
     indices_to_print <- NULL
@@ -349,6 +391,7 @@ print.summary_reg_fit_mediation <- function(x, digits = max(3, getOption("digits
                                             signif.stars = getOption("show.signif.stars"),
                                             signif.legend = signif.stars, ...) {
   # initializations
+  p_x <- length(x$x)
   p_m <- length(x$m)
   # information on variables
   print_info(x, ...)
@@ -369,10 +412,11 @@ print.summary_reg_fit_mediation <- function(x, digits = max(3, getOption("digits
   print(x$fit_ymx, digits = digits, signif.stars = signif.stars,
         signif.legend = FALSE, ...)
   # print summary of total and direct effects of x on y
-  cat("---\nTotal effect of x on y:\n")
+  plural <- if (p_x == 1L) "" else "s"
+  cat(sprintf("---\nTotal effect%s of x on y:\n", plural))
   printCoefmat(x$total, digits = digits, signif.stars = signif.stars,
                signif.legend = FALSE, ...)
-  cat("\nDirect effect of x on y:\n")
+  cat(sprintf("\nDirect effect%s of x on y:\n", plural))
   printCoefmat(x$direct, digits = digits, signif.stars = signif.stars,
                signif.legend = FALSE, ...)
   # return object invisibly
@@ -390,7 +434,7 @@ print.summary_cov_fit_mediation <- function(x, digits = max(3, getOption("digits
   printCoefmat(x$a, digits = digits, signif.stars = signif.stars,
                signif.legend = FALSE, ...)
   # print summary of direct effect of m on y
-  cat("\nDirect effect of m on y:\n")
+  cat("\nPartial effect of m on y:\n")
   printCoefmat(x$b, digits = digits, signif.stars = signif.stars,
                signif.legend = FALSE, ...)
   # print summary of total and direct effects of x on y
@@ -427,24 +471,79 @@ print.summary_test_mediation <- function(x, digits = max(3, getOption("digits")-
   invisible(x)
 }
 
+
+## internal function to print information on labels for indirect effects for
+## serial multiple mediator models
+# This now expects an object of class "fit_mediation".  In future versions,
+# this could also be turned into a generic function if necessary.
+print_indirect_info <- function(object, labels = NULL, ...) {
+  # initializations
+  if (is.null(labels)) return()  # nothing to print if there are no labels
+  x <- object$x
+  p_x <- length(x)
+  m <- object$m
+  p_m <- length(m)
+  y <- object$y
+  model <- object$model  # only implemented for regression fit
+  # label for independent variable
+  x_label <- if (p_x == 1L) x else "x"
+  # get information on indirect effect paths
+  if (model == "serial") {
+    # currently only implemented for two or three hypothesized mediators
+    if (p_m == 2L) {
+      # format strings of relevant variable names to be of equal length
+      format_m <- format(m)
+      format_ym <- format(c(y, m[2L]))
+      # generate strings describing the paths
+      paths <- c(paste(x_label, format_m, format_ym[1L], sep = " -> "),
+                 paste(x_label, format_m[1L], format_ym[2L], y, sep = " -> "))
+    } else {
+      # format strings of relevant variable names to be of equal length
+      format_m <- format(m)
+      format_ym23 <- format(c(y, m[-1L]))
+      format_ym3 <- format(c(y, m[3L]))
+      # generate strings describing the paths
+      paths <- c(paste(x_label, format_m, format_ym23[1L],
+                       sep = " -> "),
+                 paste(x_label, format_m[c(1L, 1L, 2L)],
+                       format_ym23[c(2L, 3L, 3L)],
+                       format_ym3[1L], sep = " -> "),
+                 paste(x_label, format_m[1L], format_ym23[2L],
+                       format_ym3[2L], y, sep = " -> "))
+    }
+  } else {
+    # format strings of relevant variable names to be of equal length
+    format_m <- format(m)
+    # generate strings describing the paths
+    paths <- paste(x_label, format_m, y, sep = " -> ")
+  }
+  # create data frame with information on indirect effect paths
+  indirect_info <- data.frame(Label = labels, Path = paths)
+  # print information on indirect effect paths
+  cat("\nIndirect effect paths:\n")
+  print(indirect_info, right = FALSE, row.names = FALSE)
+}
+
 ## internal function to print information on contrast definitions
 # This now expects an object of class "fit_mediation".  In future versions,
 # this could also be turned into a generic function if necessary.
-print_contrast_info <- function(object, prefix = FALSE, ...) {
+print_contrast_info <- function(object, labels = NULL, ...) {
   # initializations
   contrast <- object$contrast              # only implemented for regression fit
   have_contrast <- is.character(contrast)  # but this always works
   # if applicable, print indirect effect contrast definitions
   if (have_contrast) {
-    # extract further information
-    x <- object$x
-    m <- object$m
-    nr_indirect <- length(x) * length(m)
+    # if no labels are supplied, get names of indirect effects
+    if (is.null(labels)) {
+      labels <- get_indirect_names(object$x, object$m,
+                                   model = object$model)
+    }
+    nr_indirect <- length(labels)
     # print information on contrast definitions
     plural <- if (nr_indirect > 2L) "s" else ""
     cat(sprintf("\nIndirect effect contrast definition%s:\n", plural))
-    contrast_info <- get_contrast_info(x, m, type = contrast, prefix = prefix)
-    print(contrast_info, row.names = FALSE)
+    contrast_info <- get_contrast_info(labels, type = contrast)
+    print(contrast_info, right = FALSE, row.names = FALSE)
   }
 }
 
